/*  @(#)86  1.5 src/com/tivoli/pd/jazn/LocalRemoteDemo/PDLRTestDemo.java, pd.jazn, am611 07/03/22 08:25:56 @(#)  */

/*
 *
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2006, 2007
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * File Name: LRTestDemo.java
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Locale;
import java.util.Properties;



import com.tivoli.pd.jadmin.PDAcl;
import com.tivoli.pd.jadmin.PDAclEntryUser;
import com.tivoli.pd.jadmin.PDAdmin;
import com.tivoli.pd.jadmin.PDAppSvrConfig;
import com.tivoli.pd.jadmin.PDAppSvrSpecLocal;
import com.tivoli.pd.jadmin.PDAppSvrSpecRemote;
import com.tivoli.pd.jadmin.PDAuthzRule;
import com.tivoli.pd.jadmin.PDDomain;
import com.tivoli.pd.jadmin.PDGroup;
import com.tivoli.pd.jadmin.PDPolicy;
import com.tivoli.pd.jadmin.PDPop;
import com.tivoli.pd.jadmin.PDProtObject;
import com.tivoli.pd.jadmin.PDProtObjectSpace;
import com.tivoli.pd.jadmin.PDServer;
import com.tivoli.pd.jadmin.PDSvrInfo;
import com.tivoli.pd.jadmin.PDTodAccessInfo;
import com.tivoli.pd.jadmin.PDUser;
import com.tivoli.pd.jazn.PDAuthorizationContext;
import com.tivoli.pd.jazn.PDPermission;
import com.tivoli.pd.jazn.PDPrincipal;
import com.tivoli.pd.jutil.PDAttrs;
import com.tivoli.pd.jutil.PDContext;
import com.tivoli.pd.jutil.PDException;
import com.tivoli.pd.jutil.PDMessage;
import com.tivoli.pd.jutil.PDMessages;
import com.tivoli.pd.jutil.PDRgyGroupName;
import com.tivoli.pd.jutil.PDRgyUserName;

/**
 *  <p>
 * This class is the engine of the LRADemo program.
 * It executes the following tasks:
 * <ol>
 * <li>Constructs the enviornment, server, objects, etc. in while the demo will run.
 * <li>Runs the demo program with various options: timing, credential cache on/off, etc.
 * <li>Tears down the envionment that was setup in step #1.
 * </ol>
 * <p>
 *
 *  For this Demo it is assumed that the user has a
 *  Distinguishing name as follows:
 *    dn=user_LRADemo,ou=austin,o=ibm,c=us
 *  Note that a suffix of c=us is configured on the ldap server
 *  then a country object was defined (c=us), an organizational object (o=ibm),
 *  and then an organizational unit object(ou=austin). This default suffix
 *  can be changed by using the configuration panel's Default Suffix entry to fit
 *  whatever suffix the user has set up for his PDUsers
 *  or the ldap server can be configured as described to use the default.
 *
 *
 */
public class PDLRTestDemo {

	/**
	 *
	 */
	static PDMessages msgs = null;
	static String testName = new String("LRDemo");
	static boolean reduceOutputFromProcessPDMessages = true;
	String userName = null;
	ArrayList plcySvrs  = null;
	Locale myLocale = null;
    //note that for our test database, users always have this suffix
	//Note that a suffix of c=us is configured on the ldap server
	// then a country object was defined, an organizational object,
	// and then an organizational unit object. This Suffix could
	// be changed to fit whatever suffix the customer is using.
	
	private static String pdGroupName      = "LRADemo_group_";
	private static String pdGroupDesc      = "LRADemo_Description_group_";
	

	 /**
	 *
	 * <ol>
	 * <li> Configures a local and remote server..this results in properties files
	 * for each server.
	 * <li> Creates a PDContext from using local server's url and initializes PDAdmin
	 * operations (PDAdmin)
	 * <li> Creates a PDUser using the name of the remote application. This user will be
	 * used for both local and remote access decisions.
	 * <li> Creates group(s) depending on input criteria and adds the user to each group.
	 * This results in a more complex credential for the user. Note that the PDPrincipal
	 * which contains the credential will grow substantially as the groups are increased.
	 * <li> Create object space and two objects
	 * <li> Create ACL and POP and attach to object space
	 * <li> Create Authzrule and attach to only the first object
	 * </ol>
	 * <p>
	 * <img src="./images/PDLRADemo1.gif">
     * @param LRADemo  Class that defines various public methods for fetching
     * the configuration parameters entered on the LRAuthzDemo2 Frame. See doc
     * on LRAuthzDemo2 for the public methods and what they provide.
	 * */
	public PDLRTestDemo(PDLRAuthzDemo2 LRADemo)throws PDtamdemoException{
		super();
		// First lets configure each of the servers local and remote
		
        // create some Array lists to hold plcy and acld server names
		// for this example we only need one name in each of the arrays
		plcySvrs = new ArrayList();
		ArrayList acldSvrs = new ArrayList();
		
		// Create the  messages.
		msgs = new PDMessages();
		

		//		 create locale for US English
		myLocale = new Locale("ENGLISH", "US");
		try {
            // populate them ..very sparsely
			// First create the Server Info then add to array
			PDSvrInfo plcySvr = new PDSvrInfo(LRADemo.getJTF_policy_server_name(), 7135, 1, myLocale, msgs);
			plcySvrs.add(plcySvr);
			PDSvrInfo acldSvr = new PDSvrInfo(LRADemo.getJTF_acld_server_name(), 7136, 1, myLocale, msgs);
			acldSvrs.add(acldSvr);
		} catch (PDException e2) {

			System.out.println("Problem allocating PDSvrInfo");
			throw new PDtamdemoException("Problem allocating PDSvrInfo");
		}
		// Now the spec for the server configuration must be defined
		// local spec will be first
		PDAppSvrSpecLocal spec = null;
		// define a group names array. Will add the local app's userid to the
		// iv-admin group so that a PDContext can be opened under this
		// url and admin functions can be performed
		ArrayList groupNames = new ArrayList();
		// iv-admin to groups names so that the server's userid will
		// have admin access. When Context is opened using just the URL
		// admin functions will be enabled.
		groupNames.add(new String("iv-admin"));
		
			try {
				String workdir = new String("file:"+LRADemo.getJTF_working_dir_text());
				spec = new PDAppSvrSpecLocal(LRADemo.getJTF_local_appName_text(), // configName
						LRADemo.getJTF_local_appName_text()+" desc", // description
						null, // password - will be randomly generated
						groupNames, // Array of group names or null
						LRADemo.getJTF_appl_server_host_text(), //machine on which server will run
						LRADemo.getJTF_local_port_int(), // port number
						plcySvrs, // array of policy servers
						acldSvrs, // array of authzServers
						true, // listen for database refresh
						1000, // poll every 1000 secs
						new URL("file:"+LRADemo.getJTF_working_dir_text()), // dir where db will be stored
												// locally
						true, // enable CERTRefresh
						myLocale, // locale
						msgs // messages returned
				);
			} catch (PDException e1) {
				System.out.println("Problem creating local spec");
				e1.printStackTrace();
				throw new PDtamdemoException("Problem creating local spec");
				
			} catch (MalformedURLException e) {
				System.out.println("Malformed URL"+"file:"+LRADemo.getJTF_working_dir_text());
				e.printStackTrace();
				throw new PDtamdemoException("Malformed URL - see console stack trace");
			}
			// now configure the local mode server
			try {
				String propfile = new String("file:"+LRADemo.getJTF_working_dir_text()+"/"+
						LRADemo.getJTF_local_appName_text()+".properties");
				String keysfile = new String("file:"+LRADemo.getJTF_working_dir_text()+"/"+
						 		LRADemo.getJTF_local_appName_text()+".ks");
				PDAppSvrConfig.configureAppSvr(LRADemo.getJTF_sec_master_text(), // admin id
						LRADemo.getJPF_sec_master_password_chararr(), // password
						PDDomain.getMgmtDomainName(), // domain to be configured to
						spec, // local spec
						new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
								LRADemo.getJTF_local_appName_text()+".properties"),
						 new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
						 		LRADemo.getJTF_local_appName_text()+".ks"), // keystoreURL
						PDAppSvrConfig.PDAPPSVRCFG_REPLACE, // action
						myLocale, //locale
						"LRDemo", // productCode
						 msgs);
			} catch (PDException e1) {
				System.out.println("Problem configuring local mode Server");
				e1.printStackTrace();
				throw new PDtamdemoException("Problem creating local mode server");
			} catch (MalformedURLException e) {
				System.out.println("Malformed URL"+"file:"+LRADemo.getJTF_working_dir_text());
				e.printStackTrace();
				throw new PDtamdemoException("Malformed URL - see console stack trace");
			}
			// defined the remote spec in prepartion for the
			// remote mode server's configuration
			PDAppSvrSpecRemote rspec = null;
			
				try {
					rspec = new PDAppSvrSpecRemote(LRADemo.getJTF_remote_appName_text(), // configName
							LRADemo.getJTF_remote_appName_text()+ "_desc", // description
							null, // password generate password randomly
							null, // Array of group names or null
							LRADemo.getJTF_appl_server_host_text(), //machine on which server will run
							LRADemo.getJTF_remote_port_int(), // port number
							plcySvrs, // array of policy servers
							acldSvrs, // array of authzServers
							true, // cert refresh
							myLocale, // locale
							msgs // messages returned
					);
				} catch (PDException e1) {
					System.out.println("Problem creating remote spec");
					e1.printStackTrace();
					return;
				}
				processPDMessages(msgs);
				//				 Configure app server using configureAppSvr() method
				try {
					PDAppSvrConfig.configureAppSvr(LRADemo.getJTF_sec_master_text(), // admin id
							LRADemo.getJPF_sec_master_password_chararr(), // password
							PDDomain.getMgmtDomainName(), // domain to be configured to
							rspec, // local spec
							new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
									LRADemo.getJTF_remote_appName_text()+".properties"),
							 new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
							 		LRADemo.getJTF_remote_appName_text()+".ks"), // keystoreURL
							PDAppSvrConfig.PDAPPSVRCFG_REPLACE, // action
							myLocale, //locale
							"LRDemo", // productCode
							 msgs);
				} catch (PDException e1) {
					System.out.println("Problem configuring remote mode Server");
					e1.printStackTrace();
					return;
				} catch (MalformedURLException e) {
					System.out.println("Malformed URL"+"file:"+LRADemo.getJTF_working_dir_text());
					e.printStackTrace();
					return;
				}
			// okay now that the servers are configured...
		    // define a user, some objects, ACLs, POPs, and authz rules
			// which will later be used in the access decision
				
			 // Initialize PDAdmin 	
				try {
					PDAdmin.initialize("LRDemo", msgs);
				} catch (PDException e2) {
					System.out.println("Failed to do admin initalize");
					e2.printStackTrace();
				}
				PDContext  ctx = null;
				// Defined a PDContext
				try {
					// use default userid in properties,since it is a member of
					// the iv-admin group
					  ctx = new PDContext(new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
							LRADemo.getJTF_local_appName_text()+".properties"));
					
				} catch (PDException e3) {
					System.out.println("Problem instantiating PDContext");
					e3.printStackTrace();
				} catch (MalformedURLException e) {
					System.out.println("Malformed URL "+"file:"+LRADemo.getJTF_working_dir_text()+"/"+
							LRADemo.getJTF_local_appName_text()+".properties");
					e.printStackTrace();
				}

				PDRgyUserName pdUserRgyNameObj = null;
				userName = new String("user_LRADemo");
				// define the Distinguishing Name for the user
				String pdUserRgyName = "cn=" + "user_" +userName
				                 + "," + LRADemo.getJTF_default_suffix_text();

				pdUserRgyNameObj = new PDRgyUserName(pdUserRgyName, // LDAP DN
						                             "first", // LDAP CN
													 "last" // LDAP SN
													 );
				
				try {
					PDUser.createUser(ctx, //PDContext
							userName, // user's name
							pdUserRgyNameObj, //User Regy Object
							"User Desc", // Description
							"password".toCharArray(), // password
							null, // no initial group
							false, // ssoUser
							true, // password policy
							msgs); //info or warning msgs output

					//   Invoke the PDUser constructor using pdUserName.
					
					processPDMessages(msgs);
					
					PDUser.setAccountValid(ctx,userName, true, msgs);
					processPDMessages(msgs);
				} catch (PDException e2) {
					System.out.println("Problem creating user " + userName);
					e2.printStackTrace();
				}
				// create groups and stick the user in them.
				String pdGroupRgyName   = null;
			    PDRgyGroupName pdGroupRgyNameObj = null;
			    ArrayList members = new ArrayList();
				members.add(new String(userName));
				for(int i=0; i<LRADemo.getJTF_num_groups_int(); i++)
				{
					pdGroupRgyNameObj =
						new PDRgyGroupName("cn="+pdGroupName+i+","+LRADemo.getJTF_default_suffix_text());
					try {
						PDGroup.createGroup(
								ctx, // PDContext
								pdGroupName+i, //pd group name
								pdGroupRgyNameObj,
								   // rgyName
								pdGroupDesc+i,
								null, //container object ...null = root
								msgs
								);
					} catch (PDException e) {
						processPDMessages(msgs);
						System.out.println("Unable to create group "+pdGroupName+i);
						e.printStackTrace();
					}
					
					try {
						PDGroup.addMembers(ctx, // PDContext
								pdGroupName+i, //pd group name)
						        members, // a list of member names
								msgs
								);
					} catch (PDException e2) {
						
						processPDMessages(msgs);
						System.out.println("Unable to add "+userName+" to group "+pdGroupName+i);
						e2.printStackTrace();
					}
					
					
					
					
					
					
				}
				
				
				 // define a PDAttrs instance
				 PDAttrs nAttrs         = null;
				 try {
				 	 // create the object space
		            PDProtObjectSpace.createProtObjectSpace(ctx, //PDContext
		            		"/objectsp_LRDemo", // object space name
		                   null, // description
		                   msgs);
		
		            processPDMessages(msgs);
		
		
		           // Create the first protected Object
		           	PDProtObject.createProtObject(ctx, // PDContext
							"/objectsp_LRDemo/object_LRDemo1", // object name
														 null, // description
														 true, // policy attachable?
														 null, // aclId - should use null
														 nAttrs, // attributes - should use null
														 msgs);
						processPDMessages(msgs);
						 // Create the second protected Object
			           	PDProtObject.createProtObject(ctx, // PDContext
								"/objectsp_LRDemo/object_LRDemo2", // object name
															 null, // description
															 true, // policy attachable?
															 null, // aclId - should use null
															 nAttrs, // attributes - should use null
															 msgs);
							processPDMessages(msgs);	
						
					} catch (PDException e3) {
						System.out.println("Problem creating objectspace and/or object "+
								"/objectsp_LRDemo/object_LRDemo1");
						e3.printStackTrace();
					}
					
					// create an ACL
					String aclName = new String("acl_LRADemo");
					try {
						PDAcl.createAcl(ctx, // PDContext
						         aclName, // Name
								 aclName+" Description", //Description
								 null, // aclEntriesUser
								 null, // aclEntriesGroup
								 null, // aclEntryAnyOther
								 null, // aclEntryUnAuth
								 nAttrs, //attributes should use null
								 msgs);
						 processPDMessages(msgs);
						
						 // create the acl entry for the user_domname
						 PDAclEntryUser entry = new PDAclEntryUser(ctx, userName, "Tr",
						 msgs);
						 processPDMessages(msgs);
						 // add the entry to the acl
						 PDAcl.setPDAclEntryUser(ctx, aclName, entry, msgs);
						 processPDMessages(msgs);
                                                 // create the acl entry for the sec_master so
                                                 //  objectspace can be shown using pdadmin -- this is optional
                                                 //  but you won't be able to get pass the objectspace if this is
                                                 //  not set. You'll get a permission denied error instead.
						 entry = new PDAclEntryUser(ctx, "sec_master", "TcmdbvaBR",
						 msgs);
						 processPDMessages(msgs);
						 // add the entry to the acl
						 PDAcl.setPDAclEntryUser(ctx, aclName, entry, msgs);
						 processPDMessages(msgs);

						 // attach acl to object space
						 // both object spaces will inherit
						 PDProtObject.attachAcl(ctx,"/objectsp_LRDemo",aclName,msgs);
						
						 processPDMessages(msgs);
					} catch (PDException e4) {
						System.out.println("Problem creating acl and attaching to objects" );
						
						e4.printStackTrace();
					}
					
					// create a POP
					
					String POPName = new String("pop_LRADemo");
		            try {
						PDPop.createPop(ctx,
								POPName,
								POPName+ " Description",
								nAttrs,
								msgs);
						processPDMessages(msgs);
						
						
						PDTodAccessInfo inTodAccessInfo = new PDTodAccessInfo(
						     PDPolicy.PDPOLICY_TOD_WEEKDAY,
						                   (long)5*60, // 5:00 AM
						                   (long)17*60, // 5:00 PM
						                   (int)PDPolicy.PDPOLICY_TIME_LOCAL,

						                   myLocale,
						                   msgs);
						processPDMessages(msgs);

						PDPop.setTodAccessInfo(ctx,
								               POPName,
						                       inTodAccessInfo,
						                       msgs);
						processPDMessages(msgs);
                        // attach to object space
                        PDProtObject.attachPop(ctx,"/objectsp_LRDemo",POPName,msgs);

                        processPDMessages(msgs);
					} catch (PDException e7) {
						System.out.println("Problem creating or attaching POP to objects");
						e7.printStackTrace();
					}
		
		
					String ruleText = "<xsl:if test='string-length(azn_engine_requested_actions) = 1 and not(contains(azn_engine_requested_actions, \"m\") )'>!TRUE!</xsl:if>";
					String failReason = "Actions were not limited to 1 or there was a m in the action list";
					String ruleName = new String("rule_LRADemo");
					
					 // SETUP - Create a PDAuthzRule within the registry
		            try {
						PDAuthzRule.createAuthzRule(ctx,
						                            ruleName,
						                            ruleName+" Description",
						                            ruleText,
						                            failReason,msgs);
						processPDMessages(msgs);
						// Invoking the PDAuthzRule constructor using pdRes1Name
						PDAuthzRule rule = new PDAuthzRule(ctx,
						                                        ruleName,
						                                        msgs);
						processPDMessages(msgs);
						// only attach this to one of the Protected objects
						PDProtObject.attachAuthzRule(ctx,"/objectsp_LRDemo/object_LRDemo1",ruleName,msgs);
						processPDMessages(msgs);
					} catch (PDException e5) {
						System.out.println("Problem creating AuthzRule and attaching it");
						e5.printStackTrace();
					}
					try {
						// replicate to all servers
						// specifically the ACLD server that will be used
						// by the remote mode PDAuthorization Context
						// when the demo is executed. Else the expected
						// results may not be seen on the first couple passes
						// until a replicate timeout pops and the pdmgrd database
						// is replicated to the acld server
					
						PDServer.replicateServer(ctx,null,msgs);
					} catch (PDException e8) {
						System.out.println("Problem replicating server");
						e8.printStackTrace();
					}
					try {
						// close the PDContext
						ctx.close();
					} catch (PDException e6) {
						System.out.println("Problem closing PDContext");
						e6.printStackTrace();
					}
				
	}
/**
	 * This method deletes all the objects, users, ACLs, POPs, and user
	 * created by the constructor.
	 *<p>
	 *@param LRADemo instance of LRAuthzDemo2. Parameters entered in the initial
	 *configuration dialog are access through this class's methods.
	 **/
	public void tearDown(PDLRAuthzDemo2 LRADemo)
	{
		PDContext  ctx = null;
		// Defined a PDContext
		try {
			// use default userid in properties,since it is a member of
			// the iv-admin group
			  ctx = new PDContext(new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
					LRADemo.getJTF_local_appName_text()+".properties"));
			
		} catch (PDException e3) {
			System.out.println("Problem instantiating PDContext");
			e3.printStackTrace();
		} catch (MalformedURLException e) {
			System.out.println("Malformed URL "+"file:"+LRADemo.getJTF_working_dir_text()+"/"+
					LRADemo.getJTF_local_appName_text()+".properties");
			e.printStackTrace();
		}
		 // detach ACL, AuthzRule and POPs
		 try {
		 	// only detach ACL and POP form objectspace
			 PDProtObject.detachAcl(ctx,"/objectsp_LRDemo",msgs);
			 PDProtObject.detachPop(ctx,"/objectsp_LRDemo",msgs);
			 PDProtObject.detachAuthzRule(ctx,"/objectsp_LRDemo/object_LRDemo1",msgs);
			
			
		} catch (PDException e1) {
			processPDMessages(msgs);
			System.out.println("Problem detaching ACLs, rules, or POPS");
			e1.printStackTrace();
		}
		// delete ACL, AuthzRule and POPs
//		 create an ACL
		try {
			String aclName = new String ("acl_LRADemo");
			    PDAcl.deleteAcl(ctx,aclName,msgs);
			String POPName = new String("pop_LRADemo");
			    PDPop.deletePop(ctx, POPName, msgs);
			String ruleName = new String("rule_LRADemo");
			    PDAuthzRule.deleteAuthzRule(ctx,ruleName,msgs);
		} catch (PDException e2) {
			processPDMessages(msgs);
			System.out.println("Problem deleting ACLs, rules, or POPS");
			e2.printStackTrace();
		}
		
		    try {
				PDProtObject.deleteProtObject(ctx,"/objectsp_LRDemo/object_LRDemo1",msgs);
				PDProtObject.deleteProtObject(ctx,"/objectsp_LRDemo/object_LRDemo2",msgs);
				PDProtObjectSpace.deleteProtObjectSpace(ctx,"/objectsp_LRDemo", msgs);
			} catch (PDException e4) {
				processPDMessages(msgs);
				System.out.println("Problem deleting ProtObjects");
				e4.printStackTrace();
			}
			
			String pdGroupRgyName   = null;
		    PDRgyGroupName pdGroupRgyNameObj = null;
		    ArrayList members = new ArrayList();
			members.add(new String(userName));
			for(int i=0; i<LRADemo.getJTF_num_groups_int(); i++)
			{
				pdGroupRgyNameObj =
					new PDRgyGroupName("cn="+pdGroupName+i+","+LRADemo.getJTF_default_suffix_text());
				try {
					PDGroup.deleteGroup(
							ctx, // PDContext
							pdGroupName+i, //pd group name
							true, //delete from registry
							msgs
							);
				} catch (PDException e) {
					processPDMessages(msgs);
					System.out.println("Unable to delete group "+pdGroupName+i);
					e.printStackTrace();
				}
			}
			
		try {
			PDUser.deleteUser(ctx,userName,true,msgs);
		} catch (PDException e5) {
			processPDMessages(msgs);
			System.out.println("Problem deleting User");
			e5.printStackTrace();
		}
		
		try {
			PDAppSvrConfig.unconfigureAppSvr(LRADemo.getJTF_sec_master_text(), // admin id
					LRADemo.getJPF_sec_master_password_chararr(), // password
			                               "Default", // domain to be configured to
			                               LRADemo.getJTF_local_appName_text(), // configName
			                               LRADemo.getJTF_appl_server_host_text(), // application server's host name
										   plcySvrs, // policy server array
										   new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
												LRADemo.getJTF_local_appName_text()+".properties"), // URL
			                               myLocale, // locale	
										   msgs);
			PDAppSvrConfig.unconfigureAppSvr(LRADemo.getJTF_sec_master_text(), // admin id
					LRADemo.getJPF_sec_master_password_chararr(), // password
			                               "Default", // domain to be configured to
			                               LRADemo.getJTF_remote_appName_text(), // configName
			                               LRADemo.getJTF_appl_server_host_text(), // application server's host name
										   plcySvrs, // policy server array
										   new URL("file:"+LRADemo.getJTF_working_dir_text()+"/"+
												LRADemo.getJTF_remote_appName_text()+".properties"), // URL
			                               myLocale, // locale	
										   msgs);
		} catch (MalformedURLException e6) {
			System.out.println("Malformed URL");
			e6.printStackTrace();
		} catch (PDException e6) {
			System.out.println("Problem with unconfiguiring remote server");
			e6.printStackTrace();
		}
		
	}
	/***
	 * This method runs the LRADemo
	 * <p>
	 * The main activities that take place are:
	 * <ol>
	 * <li> Modifies the properties files for both local and remote servers
	 * to reflect credential cache setting.
	 * <li> Creates a PDAuthorizationContext from local and remote servers.
	 * operations (PDAdmin)
	 * <li> Creates Permissions, four in all, one set of two for the local
	 * servers testing of the two objects, and another set for the remote server's
	 * use.
	 * <li> Creates PDPrincipals, one for local and one for remote.
	 * <li> Excutes the following the number of times pass in by repetition parameter
	 * <ul>
	 * <li> Test access to objects by user (local server).  Both decisions should
	 * pass since Authzrules are not processed in local mode.
	 * <li> Test access to objects by user (remote server). Expect the first decision to
	 * fail since the rule says there should only be one request action not two. Second
	 * rule should pass since there is no AuthzRule on that object.
	 * <li> Allocates new PDPrincipals if parm set
	 * <li> Allocates new PDAuthorizationContext if parm newPDAuthContext true, will also by default
	 * allocate new PDPrincipals and PDPermissions since there is a new context.
	 * </ul>
	 * </ol>
	 * <p>
	 *
     * @param reps Number of times to execute access decisions
     * @param disableCache Disable the credential cache
     * @param meas_avg_time measure times for access decisions and PDPrincipal creation.
     * @param newPDPrincipal create new PDPrincipals each time through the loop
     * @param newPDAuthContext create new PDAuthorizationContexts for each iteration.
     * @param localAppProp local apps properties file spec minus the "file:" prefix
     * @param remoteAppProp remote apps properties file spec minus the "file:" prefix
     *
	 * */
	public void runDemo( int reps,
			boolean disableCache,
			boolean meas_avg_time,
			boolean newPDPrincipal,
			boolean newPDAuthContext,
			String localAppProp, String remoteAppProp )
	{
		System.out.println("---- Demo Run "+reps+" Repetitions using");
		System.out.println("                     Disable cache = "+disableCache);
		System.out.println("                     Measure time  = "+meas_avg_time);
		System.out.println("       New PDPrincipalDisable /rep = "+newPDPrincipal);
		System.out.println("   New PDAuthorizationContext /rep = "+newPDAuthContext);
		System.out.println("Times are in milliseconds");
		
		ArrayList resultsArr = null;
		// first check for cache being disabled
		Properties cfg_local_Props = new Properties();
		Properties cfg_remote_Props = new Properties();
		FileOutputStream os= null;
		// timer used for..what else timing operations!
		PDTimer t1 = new PDTimer();
		if(disableCache)
		{
			try {
				cfg_local_Props.load(new FileInputStream(localAppProp));
			} catch (FileNotFoundException e) {
				System.out.println("file not found "+localAppProp);
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("Had trobule reading file "+localAppProp);
				e.printStackTrace();
			}
			// get the cred_cache_life
			String cred_cache_life = cfg_local_Props.getProperty("appsvr-credcache-life");
			// if the value is not equal to 0 then must make it 0 to disable cache
			if(Integer.parseInt(cred_cache_life) != 0)
			{
			  cfg_local_Props.setProperty("appsvr-credcache-life","0");	
			}
			
			try {
				os = new FileOutputStream(new File(localAppProp));
			} catch (RuntimeException e1) {
				System.out.println("Could not open local properties file for output");
				e1.printStackTrace();
			} catch (FileNotFoundException e) {
				System.out.println("could not find "+localAppProp);
				e.printStackTrace();
			}
			try {
				cfg_local_Props.store(os,"LRAuthzDemo");
			} catch (IOException e2) {
				
				System.out.println("Problem writing out local App properties file");
				e2.printStackTrace();
			}

			try {
				cfg_remote_Props.load(new FileInputStream(remoteAppProp));
			} catch (FileNotFoundException e) {
				System.out.println("file not found "+remoteAppProp);
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("Had trobule reading file "+remoteAppProp);
				e.printStackTrace();
			}
			// get the cred_cache_life
			 cred_cache_life = cfg_remote_Props.getProperty("appsvr-credcache-life");
			// if the value is not equal to 0 then must make it 0 to disable cache
			if(Integer.parseInt(cred_cache_life) != 0)
			{
			  cfg_remote_Props.setProperty("appsvr-credcache-life","0");	
			}
			
			try {
				os = new FileOutputStream(new File(remoteAppProp));
			} catch (RuntimeException e1) {
				System.out.println("Could not open remote properties file for output");
				e1.printStackTrace();
			} catch (FileNotFoundException e) {
				System.out.println("could not find "+remoteAppProp);
				e.printStackTrace();
			}
			try {
				cfg_remote_Props.store(os,"LRAuthzDemo");
			} catch (IOException e2) {
				
				System.out.println("Problem writing out remote App properties file");
				e2.printStackTrace();
			}
		}
		else
		{
			try {
				cfg_local_Props.load(new FileInputStream(localAppProp));
			} catch (FileNotFoundException e) {
				System.out.println("file not found "+localAppProp);
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("Had trobule reading file "+localAppProp);
				e.printStackTrace();
			}
			// get the cred_cache_life
			String cred_cache_life = cfg_local_Props.getProperty("appsvr-credcache-life");
			// if the value is not equal to 0 then must make it 0 to disable cache
			if(Integer.parseInt(cred_cache_life) == 0)
			{
			  cfg_local_Props.setProperty("appsvr-credcache-life","7200");	
			}
			
			try {
				os = new FileOutputStream(new File(localAppProp));
			} catch (RuntimeException e1) {
				System.out.println("Could not open local properties file for output");
				e1.printStackTrace();
			} catch (FileNotFoundException e) {
				System.out.println("could not find "+localAppProp);
				e.printStackTrace();
			}
			try {
				cfg_local_Props.store(os,"LRAuthzDemo");
			} catch (IOException e2) {
				
				System.out.println("Problem writing out properties file");
				e2.printStackTrace();
			}

			try {
				cfg_remote_Props.load(new FileInputStream(remoteAppProp));
			} catch (FileNotFoundException e) {
				System.out.println("file not found "+remoteAppProp);
				e.printStackTrace();
			} catch (IOException e) {
				System.out.println("Had trobule reading file "+remoteAppProp);
				e.printStackTrace();
			}
			// get the cred_cache_life
			 cred_cache_life = cfg_remote_Props.getProperty("appsvr-credcache-life");
			// if the value is not equal to 0 then must make it 0 to disable cache
			if(Integer.parseInt(cred_cache_life) == 0)
			{
			  cfg_remote_Props.setProperty("appsvr-credcache-life","7200");	
			}
			
			try {
				os = new FileOutputStream(new File(remoteAppProp));
			} catch (RuntimeException e1) {
				System.out.println("Could not open remote properties file for output");
				e1.printStackTrace();
			} catch (FileNotFoundException e) {
				System.out.println("could not find "+remoteAppProp);
				e.printStackTrace();
			}
			try {
				cfg_remote_Props.store(os,"LRAuthzDemo");
			} catch (IOException e2) {
				System.out.println("Problem writing out properties file");
				e2.printStackTrace();
			}	
			
			
		}
		// Open up initial PDAuthorization Contexts
		PDAuthorizationContext l_ctx = null;
		PDAuthorizationContext r_ctx = null;
		try {
			
			
			l_ctx = new PDAuthorizationContext(new URL("file:"+localAppProp));
			r_ctx = new PDAuthorizationContext(new URL("file:"+remoteAppProp));
			
		} catch (MalformedURLException e) {
			System.out.println("Bad url for either local or remote loacl ="+localAppProp+
					 " remote ="+remoteAppProp);
			e.printStackTrace();
		} catch (PDException e) {
			System.out.println("Problem defining Authorization Contexts");
			e.printStackTrace();
		}
		PDPermission perm_l1 = null;
		PDPermission perm_l2 = null;
		PDPermission perm_r1 = null;
		PDPermission perm_r2 = null;
		try {
			perm_l1 = new PDPermission(l_ctx, // PDAuthorizationContext
					"/objectsp_LRDemo/object_LRDemo1", // resource name
					"Tr" // permissions
					);
			perm_l2 = new PDPermission(l_ctx, // PDAuthorizationContext
					"/objectsp_LRDemo/object_LRDemo2", // resource name
					"Tr" // permissions
					);
			perm_r1 = new PDPermission(r_ctx, // PDAuthorizationContext
					"/objectsp_LRDemo/object_LRDemo1", // resource name
					"Tr" // permissions
					);
			perm_r2 = new PDPermission(r_ctx, // PDAuthorizationContext
					"/objectsp_LRDemo/object_LRDemo2", // resource name
					"Tr" // permissions
					);
		} catch (PDException e1) {
			System.out.println("Problem creating permissions");
			e1.printStackTrace();
		}
		PDPrincipal l_PDprin = null;
		PDPrincipal r_PDprin = null;
		try {
			if(meas_avg_time)
			{
		     t1.start();
		     l_PDprin = new PDPrincipal(l_ctx,userName,"password".toCharArray());
		     t1.stop();
		     System.out.println("Time to define PDPrincipal local = "+t1.getTime());
		
		     t1.start();
		     r_PDprin = new PDPrincipal(r_ctx,userName,"password".toCharArray());
		     t1.stop();
		     System.out.println("Time to define PDPrincipal remote ="+t1.getTime());
		     byte[] pac1 = l_PDprin.getPAC(r_ctx);
		     byte[] pac2 = r_PDprin.getPAC(r_ctx);
		     System.out.println("PDPrincipal size (local) = "+ pac1.length);
		     System.out.println("PDPrincipal size (remote) = "+ pac2.length);
			}
			else
			{
			 l_PDprin = new PDPrincipal(l_ctx,userName,"password".toCharArray());
			 r_PDprin = new PDPrincipal(r_ctx,userName,"password".toCharArray());
			}
		} catch (PDException e2) {
			System.out.println("Problem creating PDPrincipal");
			e2.printStackTrace();
		}
		if(meas_avg_time)
		{
			
			
			for(int i=1; i<reps+1; i++)
			{ // for all four possibilities
		      // measure the time while doing the access decision
			System.out.println("-- rep "+i);	
			  try {
				t1.start();
				  boolean res = perm_l1.implies(l_ctx,l_PDprin);
				  t1.stop();
				  if(res)
				  {
				  	System.out.println(userName + "(local) has Tr permission to /objectsp_LRDemo/object_LRDemo1, time ="+t1.getTime());
				  }
				  else
				  {
				  	System.out.println(userName + "(local) does not have Tr permission to /objectsp_LRDemo/object_LRDemo1, time ="+t1.getTime());
				  }
				  t1.start();
				  res = perm_l2.implies(l_ctx,l_PDprin);
				  t1.stop();
				  if(res)
				  {
				  	System.out.println(userName + "(local) has Tr permission to /objectsp_LRDemo/object_LRDemo2, time ="+t1.getTime());
				  }
				  else
				  {
				  	System.out.println(userName + "(local) does not have Tr permission to /objectsp_LRDemo/object_LRDemo2, time ="+t1.getTime());
				  }
				  t1.start();
				  res = perm_r1.implies(r_ctx,r_PDprin);
				  t1.stop();
				  if(res)
				  {
				  	System.out.println(userName + "(remote) has Tr permission to /objectsp_LRDemo/object_LRDemo1, time ="+t1.getTime());
				  }
				  else
				  {
				  	System.out.println(userName + "(remote) does not have Tr permission to /objectsp_LRDemo/object_LRDemo1, time ="+t1.getTime());
				  }
				  t1.start();
				  res = perm_r2.implies(r_ctx,r_PDprin);
				  t1.stop();
				  if(res)
				  {
				  	System.out.println(userName + "(remote) has Tr permission to /objectsp_LRDemo/object_LRDemo2, time ="+t1.getTime());
				  }
				  else
				  {
				  	System.out.println(userName + "(remote) does not have Tr permission to /objectsp_LRDemo/object_LRDemo2, time ="+t1.getTime());
				  }
			} catch (PDException e3) {
				System.out.println("Problem doing the access decision");
				e3.printStackTrace();
			}
			  if(newPDAuthContext)
			  {
			  	try {
			  	// close previous PDAuthorizationContext instances
			  	l_ctx.close();
			  	r_ctx.close();
			  	// create a new one
				l_ctx = new PDAuthorizationContext(new URL("file:"+localAppProp));
				r_ctx = new PDAuthorizationContext(new URL("file:"+remoteAppProp));
			    } catch (MalformedURLException e) {
				System.out.println("Bad url for either local or remote loacl ="+localAppProp+
						 " remote ="+remoteAppProp);
				e.printStackTrace();
			    } catch (PDException e) {
				System.out.println("Problem defining Authorization Contexts");
				e.printStackTrace();
			    }
			    // since we're using new contexts
			    // need to redefine the permissions using them
			    try {
					perm_l1 = new PDPermission(l_ctx, // PDAuthorizationContext
							"/objectsp_LRDemo/object_LRDemo1", // resource name
							"Tr" // permissions
							);
					perm_l2 = new PDPermission(l_ctx, // PDAuthorizationContext
							"/objectsp_LRDemo/object_LRDemo2", // resource name
							"Tr" // permissions
							);
					perm_r1 = new PDPermission(r_ctx, // PDAuthorizationContext
							"/objectsp_LRDemo/object_LRDemo1", // resource name
							"Tr" // permissions
							);
					perm_r2 = new PDPermission(r_ctx, // PDAuthorizationContext
							"/objectsp_LRDemo/object_LRDemo2", // resource name
							"Tr" // permissions
							);
				} catch (PDException e1) {
					System.out.println("Problem creating permissions");
					e1.printStackTrace();
				}
			
			  }
			 if(newPDPrincipal)
			 {
			 	try {
					t1.start();
					 l_PDprin = new PDPrincipal(l_ctx,userName,"password".toCharArray());
					 t1.stop();
					 System.out.println("Time to define PDPrincipal local = "+t1.getTime());
					 t1.start();
					 r_PDprin = new PDPrincipal(r_ctx,userName,"password".toCharArray());
					 t1.stop();
					 System.out.println("Time to define PDPrincipal remote = "+t1.getTime());
				} catch (PDException e4) {
					System.out.println("Problem creating PDPrincipals");
					e4.printStackTrace();
				}
			 }
			}
		}
		else
		{
			for(int i=1; i<reps+1; i++)
			{
				System.out.println("-- rep "+i);
				try {
					
					  boolean res = perm_l1.implies(l_ctx,l_PDprin);
					
					  if(res)
					  {
					  	System.out.println(userName + "(local) has Tr permission to /objectsp_LRDemo/object_LRDemo1");
					  }
					  else
					  {
					  	System.out.println(userName + "(local) does not have Tr permission to /objectsp_LRDemo/object_LRDemo1");
					  }
					
					  res = perm_l2.implies(l_ctx,l_PDprin);
					
					  if(res)
					  {
					  	System.out.println(userName + "(local) has Tr permission to /objectsp_LRDemo/object_LRDemo2");
					  }
					  else
					  {
					  	System.out.println(userName + "(local) does not have Tr permission to /objectsp_LRDemo/object_LRDemo2");
					  }
					
					  res = perm_r1.implies(r_ctx,r_PDprin);
					
					  if(res)
					  {
					  	System.out.println(userName + "(remote) has Tr permission to /objectsp_LRDemo/object_LRDemo1");
					  }
					  else
					  {
					  	System.out.println(userName + "(remote) does not have Tr permission to /objectsp_LRDemo/object_LRDemo1");
					  }
					
					  res = perm_r2.implies(r_ctx,r_PDprin);
					
					  if(res)
					  {
					  	System.out.println(userName + "(remote) has Tr permission to /objectsp_LRDemo/object_LRDemo2");
					  }
					  else
					  {
					  	System.out.println(userName + "(remote) does not haveTr permission to /objectsp_LRDemo/object_LRDemo2");
					  }
				} catch (PDException e3) {
					System.out.println("Problem doing the access decision");
					e3.printStackTrace();
				}
				if(newPDAuthContext)
				  {
				  	try {
				  	// close previous PDAuthorizationContext instances
				  	l_ctx.close();
				  	r_ctx.close();
				  	// create a new one
					l_ctx = new PDAuthorizationContext(new URL(localAppProp));
					r_ctx = new PDAuthorizationContext(new URL(remoteAppProp));
				    } catch (MalformedURLException e) {
					System.out.println("Bad url for either local or remote loacl ="+localAppProp+
							 " remote ="+remoteAppProp);
					e.printStackTrace();
				    } catch (PDException e) {
					System.out.println("Problem defining Authorization Contexts");
					e.printStackTrace();
				    }
				
				
				  }
				 if(newPDPrincipal)
				 {
				 	try {
						
						 l_PDprin = new PDPrincipal(l_ctx,userName,"password".toCharArray());
						 r_PDprin = new PDPrincipal(r_ctx,userName,"password".toCharArray());
						
						
					} catch (PDException e4) {
						System.out.println("Problem creating PDPrincipals");
						e4.printStackTrace();
					}
				 }
			
			}
		}
//		 close previous PDAuthorizationContext instances
	  	try {
			l_ctx.close();
			r_ctx.close();
		} catch (PDException e3) {
			System.out.println("Problem closing PDAuthorizationContexts");
			e3.printStackTrace();
		}
		System.out.println(" ----------- end of run -------------");
		
	}
	
	
	
	/***************************************************************************
	 *
	 * The processPDMessages() method displays the contents of a PDMessages
	 * object. Upon completion, it clears the contents of the PDMessages object.
	 *
	 **************************************************************************/

	protected static void processPDMessages(PDMessages msgs) {

		boolean failureFlag = false;

		if (!reduceOutputFromProcessPDMessages) {
			banner("processPDMessages()");
		}

		try {
			if (msgs == null) {
				if (!reduceOutputFromProcessPDMessages) {
					System.out.println("NULL PDMessages object received.\n");
				}
				return;
			}

			if (msgs.size() == 0) {
				if (!reduceOutputFromProcessPDMessages) {
					System.out.println("PDMessages contained no messages.\n");
				}
				return;
			} else {
				if (!reduceOutputFromProcessPDMessages) {
					System.out.println("PDMessages contained the following "
							+ "messages:");
				}
				for (int i = 0; i < msgs.size(); i++) {
					PDMessage msg = (PDMessage) msgs.get(i);
					if (msg.getMsgSeverity() == PDMessage.PDMESSAGE_SEVERITY_ERROR) {
						failureFlag = true;
						if (!reduceOutputFromProcessPDMessages) {
							System.out.println("***PDMessage below caused the "
									+ "test method failure***\n");
						}
					}
					if (!reduceOutputFromProcessPDMessages) {
						System.out.println(msg);
					}
				}
			}

			if (failureFlag == true) {
				System.out
						.println("\nprocessPDMessages(): TEST METHOD FAILURE !!!\n");
			}

			System.out.println();
			msgs.clear();

		} catch (Exception e) {

			System.out.println(testName + ": This exception was generated "
					+ "by processPDMessages()\n");
			e.printStackTrace();
		}
	}
	
	protected static void banner(String text) {
		System.out.println();
		System.out.println("************************************************");
		System.out.println("*");
		System.out.println("*  " + testName + ":  " + text);
		System.out.println("*");
		System.out.println("************************************************");
	}
}
